@@ -90,6 +90,7 @@ typedef struct node_record node_record_t;
 #define DEFAULT_EXT_SENSORS_TYPE    "ext_sensors/none"
 #define DEFAULT_FIRST_JOB_ID        1
 #define DEFAULT_GET_ENV_TIMEOUT     2
+#define DEFAULT_GETNAMEINFO_CACHE_TIMEOUT 60
 #define DEFAULT_GROUP_TIME          600
 #define DEFAULT_GROUP_FORCE         1	/* if set, update group membership
 					 * info even if no updates to
@@ -108,8 +109,6 @@ typedef struct node_record node_record_t;
 #define DEFAULT_CORE_SPEC_PLUGIN    "core_spec/none"
 #define DEFAULT_ENFORCE_PART_LIMITS 0
 #define DEFAULT_JOB_COMP_TYPE       "jobcomp/none"
-#define DEFAULT_JOB_COMP_LOC        "/var/log/slurm_jobcomp.log"
-#define DEFAULT_JOB_COMP_DB         "slurm_jobcomp_db"
 #if defined HAVE_NATIVE_CRAY
 #  define DEFAULT_ALLOW_SPEC_RESOURCE_USAGE 1
 #  define DEFAULT_JOB_CONTAINER_PLUGIN  "job_container/cncu"
@@ -121,10 +120,10 @@ typedef struct node_record node_record_t;
 #define DEFAULT_KILL_ON_BAD_EXIT    0
 #define DEFAULT_KILL_TREE           0
 #define DEFAULT_KILL_WAIT           30
-#define DEFAULT_LAUNCH_TYPE         "launch/slurm"
 #define DEFAULT_MAIL_PROG           "/bin/mail"
 #define DEFAULT_MAIL_PROG_ALT       "/usr/bin/mail"
 #define DEFAULT_MAX_ARRAY_SIZE      1001
+#define DEFAULT_MAX_BATCH_REQUEUE   5
 #define DEFAULT_MAX_DBD_MSGS        10000
 #define DEFAULT_MAX_JOB_COUNT       10000
 #define DEFAULT_MAX_JOB_ID          0x03ff0000
@@ -192,6 +191,8 @@ typedef struct node_record node_record_t;
 #define DEFAULT_WAIT_TIME           0
 #  define DEFAULT_TREE_WIDTH        50
 #define DEFAULT_UNKILLABLE_TIMEOUT  60 /* seconds */
+#define DEFAULT_BATCH_SCRIPT_LIMIT (4 * 1024 * 1024) /* 5MB */
+#define MAX_BATCH_SCRIPT_SIZE (512 * 1024 * 1024) /* 512MB */
 
 /* MAX_TASKS_PER_NODE is defined in slurm.h
  */
@@ -261,12 +262,13 @@ typedef struct slurm_conf_partition {
 				 * NULL indicates all */
 	uint8_t disable_root_jobs; /* if set then user root can't run jobs
 				    * if NO_VAL8, use global default */
-	uint16_t exclusive_user; /* 1 if node allocations by user */
+	bool exclusive_user; /* true if node allocations by user */
 	uint32_t grace_time;	/* default grace time for partition */
 	bool     hidden_flag;	/* 1 if hidden by default */
 	List job_defaults_list;	/* List of job_defaults_t elements */
 	bool     lln_flag;	/* 1 if nodes are selected in LLN order */
 	uint32_t max_cpus_per_node; /* maximum allocated CPUs per node */
+	uint32_t max_cpus_per_socket; /* maximum allocated CPUs per socket */
 	uint16_t max_share;	/* number of jobs to gang schedule */
 	uint32_t max_time;	/* minutes or INFINITE */
 	uint64_t max_mem_per_cpu; /* maximum MB memory per allocated CPU */
@@ -276,6 +278,8 @@ typedef struct slurm_conf_partition {
 	char 	*nodes;		/* comma delimited list names of nodes */
 	uint16_t over_time_limit; /* job's time limit can be exceeded by this
 				   * number of minutes before cancellation */
+	bool power_down_on_idle; /* true if nodes POWER_DOWN upon returning to
+				  * IDLE */
 	uint16_t preempt_mode;	/* See PREEMPT_MODE_* in slurm/slurm.h */
 	uint16_t priority_job_factor;	/* job priority weight factor */
 	uint16_t priority_tier;	/* tier for scheduling and preemption */
@@ -370,6 +374,50 @@ extern int job_defaults_unpack(void **out, uint16_t protocol_version,
  */
 extern int set_nodes_alias(const char *alias_list);
 
+/* Send conf_hashtbl to the stepd */
+extern int read_conf_send_stepd(int fd);
+
+/* Receive conf_hashtbl from the slurmd */
+extern void read_conf_recv_stepd(int fd);
+
+/*
+ * Allocate memory for a config_key_pair_t pointer and initialize it by
+ * duplicating the key-value arguments. Append the resulting pair to the
+ * argument list if it's not NULL.
+ *
+ * IN: list_t *key_pair_list
+ * IN: char *key
+ * IN: char *value
+ */
+extern void read_config_add_key_pair(list_t *key_pair_list,
+				     char *key,
+				     char *value);
+
+
+/*
+ * slurm_conf_init_stepd - Since the stepd does not read in the file and
+ * receives it from the slurm we need to call a different function to do this.
+ */
+extern void slurm_conf_init_stepd(void);
+
+/*
+ * slurm_conf_init_load - load the slurm configuration from the a file.
+ * IN file_name - name of the slurm configuration file to be read
+ *	If file_name is NULL, then this routine tries to use
+ *	the value in the SLURM_CONF env variable.  Failing that,
+ *	it uses the compiled-in default file name.
+ *	If the conf structures have already been initialized by a call to
+ *	slurm_conf_init, any subsequent calls will do nothing until
+ *	slurm_conf_destroy is called.
+ * IN load_auth - If true, load the auth and hash plugins.
+ *      NOTE: false should only be passed to this if doing unit testing where we
+ *            know we are suppose to load any plugins.
+ * RET SLURM_SUCCESS if conf file is initialized.  If the slurm conf
+ *       was already initialied, return SLURM_ERROR.
+ *
+ */
+extern int slurm_conf_init_load(const char *file_name, bool load_auth);
+
 /*
  * slurm_conf_init - load the slurm configuration from the a file.
  * IN file_name - name of the slurm configuration file to be read
@@ -531,13 +579,6 @@ extern char *slurm_conf_get_aliased_nodename(void);
  */
 extern char *slurm_conf_get_bcast_address(const char *node_name);
 
-/*
- * slurm_conf_get_port - Return the port for a given NodeName
- *
- * NOTE: Caller must NOT be holding slurm_conf_lock().
- */
-extern uint16_t slurm_conf_get_port(const char *node_name);
-
 /*
  * slurm_conf_get_addr - Return the slurm_addr_t for a given NodeName in
  *	the parameter "address".  The return code is SLURM_SUCCESS on success,
@@ -548,31 +589,6 @@ extern uint16_t slurm_conf_get_port(const char *node_name);
 extern int slurm_conf_get_addr(const char *node_name, slurm_addr_t *address,
 			       uint16_t flags);
 
-/*
- * slurm_conf_get_cpus_bsct -
- * Return the cpus, boards, sockets, cores, and threads configured for a
- * given NodeName
- * Returns SLURM_SUCCESS on success, SLURM_ERROR on failure.
- *
- * NOTE: Caller must NOT be holding slurm_conf_lock().
- */
-extern int slurm_conf_get_cpus_bsct(const char *node_name,
-				    uint16_t *cpus, uint16_t *boards,
-				    uint16_t *sockets, uint16_t *cores,
-				    uint16_t *threads);
-
-/*
- * slurm_conf_get_res_spec_info - Return resource specialization info
- * for a given NodeName
- * Returns SLURM_SUCCESS on success, SLURM_ERROR on failure.
- *
- * NOTE: Caller must NOT be holding slurm_conf_lock().
- */
-extern int slurm_conf_get_res_spec_info(const char *node_name,
-					char **cpu_spec_list,
-					uint16_t *core_spec_cnt,
-					uint64_t *mem_spec_limit);
-
 /*
  * Parse slurm.conf NodeName line and return single slurm_conf_node_t*.
  *
@@ -712,4 +728,11 @@ extern void slurm_conf_add_node(node_record_t *node_ptr);
  */
 extern void slurm_conf_remove_node(char *node_name);
 
+#ifdef HAVE_FRONT_END
+/*
+ * Return the frontend port for the given hostname.
+ */
+extern uint16_t slurm_conf_get_frontend_port(char *node_hostname);
+#endif
+
 #endif /* !_READ_CONFIG_H */
