@@ -132,6 +132,12 @@ static int _setup_resv_limits(slurmdb_reservation_rec_t *resv,
 		xstrfmtcat(*extra, ", tres='%s'", resv->tres_str);
 	}
 
+	if (resv->comment) {
+		xstrcat(*cols, ", comment");
+		xstrfmtcat(*vals, ", '%s'", resv->comment);
+		xstrfmtcat(*extra, ", comment='%s'", resv->comment);
+	}
+
 	return SLURM_SUCCESS;
 }
 static int _setup_resv_cond_limits(slurmdb_reservation_cond_t *resv_cond,
@@ -334,7 +340,8 @@ extern int as_mysql_modify_resv(mysql_conn_t *mysql_conn,
 		"nodelist",
 		"node_inx",
 		"flags",
-		"tres"
+		"tres",
+		"comment",
 	};
 	enum {
 		RESV_ASSOCS,
@@ -346,6 +353,7 @@ extern int as_mysql_modify_resv(mysql_conn_t *mysql_conn,
 		RESV_NODE_INX,
 		RESV_FLAGS,
 		RESV_TRES,
+		RESV_COMMENT,
 		RESV_COUNT
 	};
 
@@ -470,7 +478,8 @@ extern int as_mysql_modify_resv(mysql_conn_t *mysql_conn,
 	if (xstrcmp(resv->assocs, row[RESV_ASSOCS]) ||
 	    (resv->flags != slurm_atoul(row[RESV_FLAGS])) ||
 	    xstrcmp(resv->nodes, row[RESV_NODE_INX]) ||
-	    xstrcmp(resv->tres_str, row[RESV_TRES]))
+	    xstrcmp(resv->tres_str, row[RESV_TRES]) ||
+	    xstrcmp(resv->comment, row[RESV_COMMENT]))
 		set = 1;
 
 	if (!resv->time_end)
@@ -609,7 +618,8 @@ extern List as_mysql_get_resvs(mysql_conn_t *mysql_conn, uid_t uid,
 		"time_start",
 		"time_end",
 		"tres",
-		"unused_wall"
+		"unused_wall",
+		"comment",
 	};
 
 	enum {
@@ -623,6 +633,7 @@ extern List as_mysql_get_resvs(mysql_conn_t *mysql_conn, uid_t uid,
 		RESV_REQ_END,
 		RESV_REQ_TRES,
 		RESV_REQ_UNUSED,
+		RESV_REQ_COMMENT,
 		RESV_REQ_COUNT
 	};
 
@@ -734,6 +745,7 @@ empty:
 		resv->flags = slurm_atoull(row[RESV_REQ_FLAGS]);
 		resv->tres_str = xstrdup(row[RESV_REQ_TRES]);
 		resv->unused_wall = atof(row[RESV_REQ_UNUSED]);
+		resv->comment = xstrdup(row[RESV_REQ_COMMENT]);
 		if (with_usage)
 			_get_usage_for_resv(
 				mysql_conn, uid, resv, row[RESV_REQ_ID]);
