@@ -57,17 +57,17 @@
 
 #include "slurm/slurm.h"
 
-#include "src/common/cli_filter.h"
+#include "src/interfaces/cli_filter.h"
 #include "src/common/cpu_frequency.h"
 #include "src/common/env.h"
-#include "src/common/gres.h"
-#include "src/common/plugstack.h"
+#include "src/interfaces/gres.h"
 #include "src/common/proc_args.h"
 #include "src/common/read_config.h"
-#include "src/common/select.h"
-#include "src/common/slurm_auth.h"
+#include "src/interfaces/select.h"
+#include "src/interfaces/auth.h"
 #include "src/common/slurm_rlimits_info.h"
 #include "src/common/slurm_time.h"
+#include "src/common/spank.h"
 #include "src/common/uid.h"
 #include "src/common/xmalloc.h"
 #include "src/common/xsignal.h"
@@ -81,11 +81,9 @@ extern pid_t getpgid(pid_t pid);
 #endif
 
 #define MAX_RETRIES	10
-#define POLL_SLEEP	3	/* retry interval in seconds  */
+#define POLL_SLEEP	0.5	/* retry interval in seconds  */
 
 char *argvzero = NULL;
-char **command_argv;
-int command_argc;
 pid_t command_pid = -1;
 char *work_dir = NULL;
 static int is_interactive;
@@ -187,8 +185,12 @@ int main(int argc, char **argv)
 	slurm_allocation_callbacks_t callbacks;
 	ListIterator iter_req, iter_resp;
 
-	slurm_conf_init(NULL);
+	slurm_init(NULL);
 	log_init(xbasename(argv[0]), logopt, 0, NULL);
+
+	if (cli_filter_init() != SLURM_SUCCESS)
+		fatal("failed to initialize cli_filter plugin");
+
 	argvzero = argv[0];
 	_set_exit_code();
 
@@ -316,31 +318,16 @@ int main(int argc, char **argv)
 			error("no controlling terminal: please set --no-shell");
 			exit(error_exit);
 		}
-#ifdef SALLOC_RUN_FOREGROUND
-	} else if ((!saopt.no_shell) && (pid == getpgrp())) {
-		if (tpgid == pid)
-			is_interactive = true;
-		while (tcgetpgrp(STDIN_FILENO) != pid) {
-			if (!is_interactive) {
-				error("Waiting for program to be placed in "
-				      "the foreground");
-				is_interactive = true;
-			}
-			killpg(pid, SIGTTIN);
-		}
-	}
-#else
 	} else if ((!saopt.no_shell) && (getpgrp() == tcgetpgrp(STDIN_FILENO))) {
 		is_interactive = true;
 	}
-#endif
 	/*
 	 * Reset saved tty attributes at exit, in case a child
 	 * process died before properly resetting terminal.
 	 */
 	if (is_interactive)
 		atexit(_reset_input_mode);
-	if (opt.gid != getgid()) {
+	if (opt.gid != SLURM_AUTH_NOBODY) {
 		if (setgid(opt.gid) < 0) {
 			error("setgid: %m");
 			exit(error_exit);
@@ -426,7 +413,7 @@ int main(int argc, char **argv)
 	}
 
 	/* If the requested uid is different than ours, become that uid */
-	if (getuid() != opt.uid) {
+	if (opt.uid != SLURM_AUTH_NOBODY) {
 		/* drop extended groups before changing uid/gid */
 		if ((setgroups(0, NULL) < 0)) {
 			error("setgroups: %m");
@@ -613,7 +600,7 @@ int main(int argc, char **argv)
 	slurm_mutex_lock(&allocation_state_lock);
 	if (suspend_flag)
 		slurm_cond_wait(&allocation_state_cond, &allocation_state_lock);
-	command_pid = _fork_command(command_argv);
+	command_pid = _fork_command(opt.argv);
 	slurm_cond_broadcast(&allocation_state_cond);
 	slurm_mutex_unlock(&allocation_state_lock);
 
@@ -634,7 +621,7 @@ int main(int argc, char **argv)
 			rc_pid = waitpid(command_pid, &status, WUNTRACED);
 		} while (WIFSTOPPED(status) || ((rc_pid == -1) && (!exit_flag)));
 		if ((rc_pid == -1) && (errno != EINTR))
-			error("waitpid for %s failed: %m", command_argv[0]);
+			error("waitpid for %s failed: %m", opt.argv[0]);
 	}
 
 	if (is_interactive)
@@ -676,7 +663,7 @@ relinquish:
 			_forward_signal(SIGKILL);
 		} else if (WIFSIGNALED(status)) {
 			verbose("Command \"%s\" was terminated by signal %d",
-				command_argv[0], WTERMSIG(status));
+				opt.argv[0], WTERMSIG(status));
 			/* if we get these signals we return a normal
 			 * exit since this was most likely sent from the
 			 * user */
@@ -694,6 +681,7 @@ relinquish:
 	}
 
 #ifdef MEMORY_LEAK_DEBUG
+	cli_filter_fini();
 	select_g_fini();
 	slurm_reset_all_options(&opt, false);
 	slurm_auth_fini();
@@ -745,8 +733,8 @@ static void _match_job_name(job_desc_msg_t *desc_last, List job_req_list)
 	if (!desc_last)
 		return;
 
-	if (!desc_last->name && command_argv[0])
-		desc_last->name = xstrdup(xbasename(command_argv[0]));
+	if (!desc_last->name && opt.argv[0])
+		desc_last->name = xstrdup(xbasename(opt.argv[0]));
 	name = desc_last->name;
 
 	if (!job_req_list)
@@ -820,6 +808,8 @@ static int _fill_job_desc_from_opts(job_desc_msg_t *desc)
 		return -1;
 
 	desc->wait_all_nodes = saopt.wait_all_nodes;
+	desc->argv = opt.argv;
+	desc->argc = opt.argc;
 
 	return 0;
 }
@@ -921,7 +911,7 @@ static void _signal_while_allocating(int signo)
 {
 	allocation_interrupted = true;
 	if (my_job_id != 0) {
-		slurm_complete_job(my_job_id, NO_VAL);
+		slurm_complete_job(my_job_id, 128 + signo);
 	}
 }
 
@@ -986,7 +976,7 @@ static void _job_complete_handler(srun_job_complete_msg_t *comp)
 			if (signal) {
 				 verbose("Sending signal %d to command \"%s\","
 					 " pid %d",
-					 signal, command_argv[0], command_pid);
+					 signal, opt.argv[0], command_pid);
 				if (suspend_flag)
 					_forward_signal(SIGCONT);
 				_forward_signal(signal);
@@ -1077,39 +1067,37 @@ static void _set_rlimits(char **env)
 /* returns 1 if job and nodes are ready for job to begin, 0 otherwise */
 static int _wait_nodes_ready(resource_allocation_response_msg_t *alloc)
 {
-	int is_ready = 0, i, rc;
-	int cur_delay = 0;
-	int max_delay;
+	double cur_delay = 0;
+	double cur_sleep = 0;
+	int is_ready = 0, i = 0, rc;
 	bool job_killed = false;
 
-	if (slurm_conf.suspend_timeout || slurm_conf.resume_timeout) {
-		max_delay = slurm_conf.suspend_timeout +
-			    slurm_conf.resume_timeout;
-		max_delay *= 5;		/* Allow for ResumeRate support */
-	} else {
-		max_delay = 300;	/* Wait to 5 min for PrologSlurmctld */
-	}
-
 	if (alloc->alias_list && !xstrcmp(alloc->alias_list, "TBD"))
 		saopt.wait_all_nodes = 1;	/* Wait for boot & addresses */
 	if (saopt.wait_all_nodes == NO_VAL16)
 		saopt.wait_all_nodes = 0;
 
-	for (i = 0; (cur_delay < max_delay); i++) {
-
-		if (i == 1) {
+	while (true) {
+		if (i) {
 			/*
-			 * Only sleep a short time on the first miss.
+			 * First sleep should be very quick to improve
+			 * responsiveness.
+			 *
+			 * Otherwise, increment by POLL_SLEEP for every loop.
 			 */
-			usleep(500); /* Not adding sub-sec to cur_delay */
-		} else if (i) {
+			if (cur_delay == 0)
+				cur_sleep = 0.1;
+			else if (cur_sleep < 300)
+				cur_sleep = POLL_SLEEP * i;
 			if (i == 2)
 				info("Waiting for resource configuration");
-			else
-				debug("still waiting");
-			sleep(POLL_SLEEP);
-			cur_delay += POLL_SLEEP;
+			else if (i > 2)
+				debug("Waited %f sec and still waiting: next sleep for %f sec",
+				      cur_delay, cur_sleep);
+			usleep(USEC_IN_SEC * cur_sleep);
+			cur_delay += cur_sleep;
 		}
+		i += 1;
 
 		rc = slurm_job_node_ready(alloc->job_id);
 		if (rc == READY_JOB_FATAL)
