@@ -41,10 +41,12 @@
 
 #include <string.h>
 
-#include "src/common/select.h"
 #include "src/common/xassert.h"
 #include "src/common/xmalloc.h"
 #include "src/common/xstring.h"
+
+#include "src/interfaces/select.h"
+
 #include "src/slurmctld/agent.h"
 #include "src/slurmctld/fed_mgr.h"
 #include "src/slurmctld/proc_req.h"
@@ -208,12 +210,25 @@ typedef struct {
 	char *node_name;
 } srun_node_fail_args_t;
 
+slurm_addr_t *_srun_set_addr(step_record_t *step_ptr)
+{
+	char *nodeaddr;
+	slurm_addr_t *addr = xmalloc(sizeof(*addr));
+
+	if ((nodeaddr = slurm_conf_get_nodeaddr(step_ptr->host))) {
+		slurm_set_addr(addr, step_ptr->port, nodeaddr);
+		xfree(nodeaddr);
+	} else
+		slurm_set_addr(addr, step_ptr->port, step_ptr->host);
+
+	return addr;
+}
+
 static int _srun_node_fail(void *x, void *arg)
 {
 	step_record_t *step_ptr = (step_record_t *) x;
 	srun_node_fail_args_t *args = (srun_node_fail_args_t *) arg;
 	slurm_addr_t *addr;
-	char *nodeaddr;
 	srun_node_fail_msg_t *msg_arg;
 
 	if (!step_ptr->step_node_bitmap)   /* pending step */
@@ -226,10 +241,7 @@ static int _srun_node_fail(void *x, void *arg)
 	if (!step_ptr->port || !step_ptr->host || (step_ptr->host[0] == '\0'))
 		return 0;
 
-	addr = xmalloc(sizeof(slurm_addr_t));
-	nodeaddr = slurm_conf_get_nodeaddr(step_ptr->host);
-	slurm_set_addr(addr, step_ptr->port, nodeaddr);
-	xfree(nodeaddr);
+	addr = _srun_set_addr(step_ptr);
 
 	msg_arg = xmalloc(sizeof(*msg_arg));
 	memcpy(&msg_arg->step_id, &step_ptr->step_id, sizeof(msg_arg->step_id));
@@ -333,7 +345,6 @@ static int _srun_step_timeout(void *x, void *arg)
 {
 	step_record_t *step_ptr = (step_record_t *) x;
 	slurm_addr_t *addr;
-	char *nodeaddr;
 	srun_timeout_msg_t *msg_arg;
 
 	xassert(step_ptr);
@@ -344,12 +355,9 @@ static int _srun_step_timeout(void *x, void *arg)
 	if (!step_ptr->port || !step_ptr->host || (step_ptr->host[0] == '\0'))
 		return 0;
 
-	addr = xmalloc(sizeof(*addr));
 	msg_arg = xmalloc(sizeof(*msg_arg));
 
-	nodeaddr = slurm_conf_get_nodeaddr(step_ptr->host);
-	slurm_set_addr(addr, step_ptr->port, nodeaddr);
-	xfree(nodeaddr);
+	addr = _srun_set_addr(step_ptr);
 
 	memcpy(&msg_arg->step_id, &step_ptr->step_id, sizeof(msg_arg->step_id));
 	msg_arg->timeout = step_ptr->job_ptr->end_time;
@@ -536,11 +544,7 @@ extern void srun_step_complete(step_record_t *step_ptr)
 
 	xassert(step_ptr);
 	if (step_ptr->port && step_ptr->host && step_ptr->host[0]) {
-		char *nodeaddr;
-		addr = xmalloc(sizeof(slurm_addr_t));
-		nodeaddr = slurm_conf_get_nodeaddr(step_ptr->host);
-		slurm_set_addr(addr, step_ptr->port, nodeaddr);
-		xfree(nodeaddr);
+		addr = _srun_set_addr(step_ptr);
 
 		msg_arg = xmalloc(sizeof(srun_job_complete_msg_t));
 		memcpy(&msg_arg->step_id, &step_ptr->step_id,
@@ -564,11 +568,7 @@ extern void srun_step_missing(step_record_t *step_ptr, char *node_list)
 
 	xassert(step_ptr);
 	if (step_ptr->port && step_ptr->host && step_ptr->host[0]) {
-		char *nodeaddr;
-		addr = xmalloc(sizeof(slurm_addr_t));
-		nodeaddr = slurm_conf_get_nodeaddr(step_ptr->host);
-		slurm_set_addr(addr, step_ptr->port, nodeaddr);
-		xfree(nodeaddr);
+		addr = _srun_set_addr(step_ptr);
 
 		msg_arg = xmalloc(sizeof(srun_step_missing_msg_t));
 		memcpy(&msg_arg->step_id, &step_ptr->step_id,
@@ -593,11 +593,7 @@ extern void srun_step_signal(step_record_t *step_ptr, uint16_t signal)
 
 	xassert(step_ptr);
 	if (step_ptr->port && step_ptr->host && step_ptr->host[0]) {
-		char *nodeaddr;
-		addr = xmalloc(sizeof(slurm_addr_t));
-		nodeaddr = slurm_conf_get_nodeaddr(step_ptr->host);
-		slurm_set_addr(addr, step_ptr->port, nodeaddr);
-		xfree(nodeaddr);
+		addr = _srun_set_addr(step_ptr);
 
 		msg_arg = xmalloc(sizeof(job_step_kill_msg_t));
 		memcpy(&msg_arg->step_id, &step_ptr->step_id,
